@@ -5,20 +5,23 @@ import os
 from abc import ABC
 from collections import defaultdict
 from pathlib import Path
-from typing import Dict, Optional, Tuple
+from typing import Dict, Optional, Sequence, Tuple
 
 import h5py
 import numpy as np
 import torch
 import wandb
-from omegaconf import DictConfig
+from omegaconf import DictConfig, OmegaConf
 from pytorch_lightning import Trainer
 from torch import nn
 from torch.utils.data import DataLoader
 from torchmetrics.metric import Metric
 
-from mridc.collections.common.parts.fft import ifft2c
-from mridc.collections.common.parts.utils import rss_complex
+from mridc.collections.common.parts.fft import fft2, ifft2
+from mridc.collections.common.parts.utils import (
+    is_none,
+    rss_complex,
+)
 from mridc.collections.reconstruction.data.mri_data import FastMRISliceDataset
 from mridc.collections.reconstruction.data.subsample import create_mask_for_mask_type
 from mridc.collections.reconstruction.metrics.evaluate import mse, nmse, psnr, ssim
@@ -28,7 +31,7 @@ from mridc.collections.reconstruction.parts.utils import batched_mask_center
 from mridc.core.classes.modelPT import ModelPT
 from mridc.utils.model_utils import convert_model_config_to_dict_config, maybe_update_config_version
 
-__all__ = ["BaseMRIReconstructionModel", "BaseSensitivityModel"]
+__all__ = ["BaseMRIReconstructionModel", "BaseSensitivityModel", "DistributedMetricSum"]
 
 
 class DistributedMetricSum(Metric):
@@ -67,6 +70,30 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
         # init superclass
         super().__init__(cfg=cfg, trainer=trainer)
 
+        cfg_dict = OmegaConf.to_container(cfg, resolve=True)
+
+        self.fft_centered = cfg_dict.get("fft_centered")
+        self.fft_normalization = cfg_dict.get("fft_normalization")
+        self.spatial_dims = cfg_dict.get("spatial_dims")
+        self.coil_dim = cfg_dict.get("coil_dim")
+
+        self.coil_combination_method = cfg_dict.get("coil_combination_method")
+
+        # Initialize the sensitivity network if use_sens_net is True
+        self.use_sens_net = cfg_dict.get("use_sens_net")
+        if self.use_sens_net:
+            self.sens_net = BaseSensitivityModel(
+                cfg_dict.get("sens_chans"),
+                cfg_dict.get("sens_pools"),
+                fft_centered=self.fft_centered,
+                fft_normalization=self.fft_normalization,
+                spatial_dims=self.spatial_dims,
+                coil_dim=self.coil_dim,
+                mask_type=cfg_dict.get("sens_mask_type"),
+                normalize=cfg_dict.get("sens_normalize"),
+                mask_center=cfg_dict.get("sens_mask_center"),
+            )
+
         self.MSE = DistributedMetricSum()
         self.NMSE = DistributedMetricSum()
         self.SSIM = DistributedMetricSum()
@@ -79,8 +106,7 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
         self.ssim_vals: Dict = defaultdict(dict)
         self.psnr_vals: Dict = defaultdict(dict)
 
-    # skipcq: PYL-R0201
-    def process_loss(self, target, pred, _loss_fn):
+    def process_loss(self, target, pred, _loss_fn, mask=None):
         """
         Processes the loss.
 
@@ -93,6 +119,8 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
             torch.Tensor, shape [batch_size, n_x, n_y, 2]
         _loss_fn: Loss function.
             torch.nn.Module, default torch.nn.L1Loss()
+        mask: Mask for the loss.
+            torch.Tensor, shape [batch_size, n_x, n_y, 2]
 
         Returns
         -------
@@ -100,14 +128,17 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
             If self.accumulate_loss is True, returns an accumulative result of all intermediate losses.
         """
         target = torch.abs(target / torch.max(torch.abs(target)))
+        pred = torch.abs(pred / torch.max(torch.abs(pred)))
+
         if "ssim" in str(_loss_fn).lower():
             max_value = np.array(torch.max(torch.abs(target)).item()).astype(np.float32)
 
             def loss_fn(x, y):
                 """Calculate the ssim loss."""
+                y = torch.abs(y / torch.max(torch.abs(y)))
                 return _loss_fn(
-                    x.unsqueeze(dim=1),
-                    torch.abs(y / torch.max(torch.abs(y))).unsqueeze(dim=1),
+                    x.unsqueeze(dim=self.coil_dim),
+                    y.unsqueeze(dim=self.coil_dim),
                     data_range=torch.tensor(max_value).unsqueeze(dim=0).to(x.device),
                 )
 
@@ -115,7 +146,8 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
 
             def loss_fn(x, y):
                 """Calculate other loss."""
-                return _loss_fn(x, torch.abs(y / torch.max(torch.abs(y))))
+                y = torch.abs(y / torch.max(torch.abs(y)))
+                return _loss_fn(x, y)
 
         return loss_fn(target, pred)
 
@@ -170,6 +202,8 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
                 torch.Tensor, shape [batch_size, n_x, n_y, 2]
             'target': target data,
                 torch.Tensor, shape [batch_size, n_x, n_y, 2]
+            'phase_shift': phase shift for simulated motion,
+                torch.Tensor
             'fname': filename,
                 str, shape [batch_size]
             'slice_idx': slice_idx,
@@ -191,8 +225,13 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
         'log': log,
             dict, shape [1]
         """
-        y, sensitivity_maps, mask, init_pred, target, _, _, acc = batch
+        kspace, y, sensitivity_maps, mask, init_pred, target, _, _, acc = batch
+
         y, mask, init_pred, r = self.process_inputs(y, mask, init_pred)
+
+        if self.use_sens_net:
+            sensitivity_maps = self.sens_net(kspace, mask)
+
         preds = self.forward(y, sensitivity_maps, mask, init_pred, target)
 
         if self.accumulate_estimates:
@@ -201,9 +240,9 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
             except StopIteration:
                 pass
 
-            train_loss = sum(self.process_loss(target, preds, _loss_fn=self.train_loss_fn))
+            train_loss = sum(self.process_loss(target, preds, _loss_fn=self.train_loss_fn, mask=None))
         else:
-            train_loss = self.process_loss(target, preds, _loss_fn=self.train_loss_fn)
+            train_loss = self.process_loss(target, preds, _loss_fn=self.train_loss_fn, mask=None)
 
         acc = r if r != 0 else acc
         tensorboard_logs = {
@@ -229,6 +268,8 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
                 torch.Tensor, shape [batch_size, n_x, n_y, 2]
             'target': target data,
                 torch.Tensor, shape [batch_size, n_x, n_y, 2]
+            'phase_shift': phase shift for simulated motion,
+                torch.Tensor
             'fname': filename,
                 str, shape [batch_size]
             'slice_idx': slice_idx,
@@ -250,8 +291,13 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
         'log': log,
             dict, shape [1]
         """
-        y, sensitivity_maps, mask, init_pred, target, fname, slice_num, _ = batch
-        y, mask, init_pred, _ = self.process_inputs(y, mask, init_pred)
+        kspace, y, sensitivity_maps, mask, init_pred, target, fname, slice_num, _ = batch
+
+        y, mask, init_pred, r = self.process_inputs(y, mask, init_pred)
+
+        if self.use_sens_net:
+            sensitivity_maps = self.sens_net(kspace, mask)
+
         preds = self.forward(y, sensitivity_maps, mask, init_pred, target)
 
         if self.accumulate_estimates:
@@ -259,10 +305,9 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
                 preds = next(preds)
             except StopIteration:
                 pass
-
-            val_loss = sum(self.process_loss(target, preds, _loss_fn=self.eval_loss_fn))
+            val_loss = sum(self.process_loss(target, preds, _loss_fn=self.eval_loss_fn, mask=None))
         else:
-            val_loss = self.process_loss(target, preds, _loss_fn=self.eval_loss_fn)
+            val_loss = self.process_loss(target, preds, _loss_fn=self.eval_loss_fn, mask=None)
 
         # Cascades
         if isinstance(preds, list):
@@ -312,6 +357,8 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
                 torch.Tensor, shape [batch_size, n_x, n_y, 2]
             'target': target data,
                 torch.Tensor, shape [batch_size, n_x, n_y, 2]
+            'phase_shift': phase shift for simulated motion,
+                torch.Tensor
             'fname': filename,
                 str, shape [batch_size]
             'slice_idx': slice_idx,
@@ -334,8 +381,13 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
         pred: Predicted data.
             torch.Tensor, shape [batch_size, n_x, n_y, 2]
         """
-        y, sensitivity_maps, mask, init_pred, target, fname, slice_num, _ = batch
-        y, mask, init_pred, _ = self.process_inputs(y, mask, init_pred)
+        kspace, y, sensitivity_maps, mask, init_pred, target, fname, slice_num, _ = batch
+
+        y, mask, init_pred, r = self.process_inputs(y, mask, init_pred)
+
+        if self.use_sens_net:
+            sensitivity_maps = self.sens_net(kspace, mask)
+
         preds = self.forward(y, sensitivity_maps, mask, init_pred, target)
 
         if self.accumulate_estimates:
@@ -352,6 +404,8 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
         if isinstance(preds, list):
             preds = preds[-1]
 
+        preds = preds / torch.abs(preds).max()  # type: ignore
+
         slice_num = int(slice_num)
         name = str(fname[0])  # type: ignore
         key = f"{name}_images_idx_{slice_num}"  # type: ignore
@@ -368,6 +422,17 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
         self.log_image(f"{key}/reconstruction", output)
         self.log_image(f"{key}/error", error)
 
+        target = target.numpy()  # type: ignore
+        output = output.numpy()  # type: ignore
+        self.mse_vals[fname][slice_num] = torch.tensor(mse(target, output)).view(1)
+        self.nmse_vals[fname][slice_num] = torch.tensor(nmse(target, output)).view(1)
+        self.ssim_vals[fname][slice_num] = torch.tensor(ssim(target, output, maxval=output.max() - output.min())).view(
+            1
+        )
+        self.psnr_vals[fname][slice_num] = torch.tensor(psnr(target, output, maxval=output.max() - output.min())).view(
+            1
+        )
+
         return name, slice_num, preds.detach().cpu().numpy()
 
     def log_image(self, name, image):
@@ -381,6 +446,11 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
         image: Image to log.
             torch.Tensor, shape [batch_size, n_x, n_y, 2]
         """
+        if image.dim() > 3:
+            image = image[0, 0, :, :].unsqueeze(0)
+        elif image.shape[0] != 1:
+            image = image[0].unsqueeze(0)
+
         if "wandb" in self.logger.__module__.lower():
             self.logger.experiment.log({name: wandb.Image(image.numpy())})
         else:
@@ -457,6 +527,48 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
         -------
         Saves the reconstructed images to .h5 files.
         """
+        # Log metrics.
+        # Taken from: https://github.com/facebookresearch/fastMRI/blob/main/fastmri/pl_modules/mri_module.py
+        mse_vals = defaultdict(dict)
+        nmse_vals = defaultdict(dict)
+        ssim_vals = defaultdict(dict)
+        psnr_vals = defaultdict(dict)
+
+        for k in self.mse_vals.keys():
+            mse_vals[k].update(self.mse_vals[k])
+        for k in self.nmse_vals.keys():
+            nmse_vals[k].update(self.nmse_vals[k])
+        for k in self.ssim_vals.keys():
+            ssim_vals[k].update(self.ssim_vals[k])
+        for k in self.psnr_vals.keys():
+            psnr_vals[k].update(self.psnr_vals[k])
+
+        # apply means across image volumes
+        metrics = {"MSE": 0, "NMSE": 0, "SSIM": 0, "PSNR": 0}
+        local_examples = 0
+        for fname in mse_vals:
+            local_examples += 1
+            metrics["MSE"] = metrics["MSE"] + torch.mean(torch.cat([v.view(-1) for _, v in mse_vals[fname].items()]))
+            metrics["NMSE"] = metrics["NMSE"] + torch.mean(
+                torch.cat([v.view(-1) for _, v in nmse_vals[fname].items()])
+            )
+            metrics["SSIM"] = metrics["SSIM"] + torch.mean(
+                torch.cat([v.view(-1) for _, v in ssim_vals[fname].items()])
+            )
+            metrics["PSNR"] = metrics["PSNR"] + torch.mean(
+                torch.cat([v.view(-1) for _, v in psnr_vals[fname].items()])
+            )
+
+        # reduce across ddp via sum
+        metrics["MSE"] = self.MSE(metrics["MSE"])
+        metrics["NMSE"] = self.NMSE(metrics["NMSE"])
+        metrics["SSIM"] = self.SSIM(metrics["SSIM"])
+        metrics["PSNR"] = self.PSNR(metrics["PSNR"])
+
+        tot_examples = self.TotExamples(torch.tensor(local_examples))
+        for metric, value in metrics.items():
+            self.log(f"{metric}", value / tot_examples)
+
         reconstructions = defaultdict(list)
         for fname, slice_num, output in outputs:
             reconstructions[fname].append((slice_num, output))
@@ -533,14 +645,15 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
         dataloader: DataLoader.
             torch.utils.data.DataLoader
         """
-        if cfg.get("dataset_type") != "FastMRI":
-            raise ValueError(f"Unknown dataset type: {cfg.get('dataset_type')}")
-
+        mask_root = cfg.get("mask_path")
         mask_args = cfg.get("mask_args")
-        mask_type = mask_args.get("type")
         shift_mask = mask_args.get("shift_mask")
+        mask_type = mask_args.get("type")
 
-        if mask_type is not None and mask_type != "None":
+        mask_func = None  # type: ignore
+        mask_center_scale = 0.02
+
+        if is_none(mask_root) and not is_none(mask_type):
             accelerations = mask_args.get("accelerations")
             center_fractions = mask_args.get("center_fractions")
             mask_center_scale = mask_args.get("scale")
@@ -550,29 +663,35 @@ class BaseMRIReconstructionModel(ModelPT, ABC):
                     create_mask_for_mask_type(mask_type, [cf] * 2, [acc] * 2)
                     for acc, cf in zip(accelerations, center_fractions)
                 ]
-                if len(accelerations) > 2
+                if len(accelerations) >= 2
                 else [create_mask_for_mask_type(mask_type, center_fractions, accelerations)]
             )
-        else:
-            mask_func = None  # type: ignore
-            mask_center_scale = 0.02
 
         dataset = FastMRISliceDataset(
             root=cfg.get("data_path"),
-            sense_root=cfg.get("sense_data_path"),
+            sense_root=cfg.get("sense_path"),
+            mask_root=cfg.get("mask_path"),
             challenge=cfg.get("challenge"),
             transform=MRIDataTransforms(
+                coil_combination_method=cfg.get("coil_combination_method"),
+                dimensionality=cfg.get("dimensionality"),
                 mask_func=mask_func,
                 shift_mask=shift_mask,
                 mask_center_scale=mask_center_scale,
+                remask=cfg.get("remask"),
                 normalize_inputs=cfg.get("normalize_inputs"),
                 crop_size=cfg.get("crop_size"),
                 crop_before_masking=cfg.get("crop_before_masking"),
                 kspace_zero_filling_size=cfg.get("kspace_zero_filling_size"),
-                fft_type=cfg.get("fft_type"),
+                fft_centered=cfg.get("fft_centered"),
+                fft_normalization=cfg.get("fft_normalization"),
+                max_norm=cfg.get("max_norm"),
+                spatial_dims=cfg.get("spatial_dims"),
+                coil_dim=cfg.get("coil_dim"),
                 use_seed=cfg.get("use_seed"),
             ),
             sample_rate=cfg.get("sample_rate"),
+            consecutive_slices=cfg.get("consecutive_slices"),
         )
         if cfg.shuffle:
             sampler = torch.utils.data.RandomSampler(dataset)
@@ -605,7 +724,10 @@ class BaseSensitivityModel(nn.Module, ABC):
         drop_prob: float = 0.0,
         padding_size: int = 15,
         mask_type: str = "2D",  # TODO: make this generalizable
-        fft_type: str = "orthogonal",
+        fft_centered: bool = True,
+        fft_normalization: str = "ortho",
+        spatial_dims: Sequence[int] = None,
+        coil_dim: int = 1,
         normalize: bool = True,
         mask_center: bool = True,
     ):
@@ -628,8 +750,14 @@ class BaseSensitivityModel(nn.Module, ABC):
             int
         mask_type: Type of mask to use.
             str
-        fft_type: Type of FFT to use.
+        fft_centered: Whether to center the FFT.
+            bool
+        fft_normalization: Type of FFT normalization to use.
             str
+        spatial_dims: Spatial dimensions of the data.
+            tuple
+        coil_dim: Coil dimension.
+            int
         normalize: Whether to normalize the input data.
             bool
         mask_center: Whether mask the center of the image.
@@ -638,7 +766,6 @@ class BaseSensitivityModel(nn.Module, ABC):
         super().__init__()
 
         self.mask_type = mask_type
-        self.fft_type = fft_type
 
         self.norm_unet = NormUnet(
             chans,
@@ -651,6 +778,10 @@ class BaseSensitivityModel(nn.Module, ABC):
         )
 
         self.mask_center = mask_center
+        self.fft_centered = fft_centered
+        self.fft_normalization = fft_normalization
+        self.spatial_dims = spatial_dims if spatial_dims is not None else [-2, -1]
+        self.coil_dim = coil_dim
         self.normalize = normalize
 
     @staticmethod
@@ -695,7 +826,7 @@ class BaseSensitivityModel(nn.Module, ABC):
         return x.view(batch_size, c, h, w, comp)
 
     @staticmethod
-    def divide_root_sum_of_squares(x: torch.Tensor) -> torch.Tensor:
+    def divide_root_sum_of_squares(x: torch.Tensor, coil_dim: int) -> torch.Tensor:
         """
         Divide the input by the root of the sum of squares of the magnitude of each complex number.
 
@@ -703,13 +834,15 @@ class BaseSensitivityModel(nn.Module, ABC):
         ----------
         x: Tensor to divide.
             torch.Tensor
+        coil_dim: Coil dimension.
+            int
 
         Returns
         -------
         RSS output tensor.
             torch.Tensor
         """
-        return x / rss_complex(x, dim=1).unsqueeze(-1).unsqueeze(1)
+        return x / rss_complex(x, dim=coil_dim).unsqueeze(-1).unsqueeze(coil_dim)
 
     @staticmethod
     def get_pad_and_num_low_freqs(
@@ -777,10 +910,17 @@ class BaseSensitivityModel(nn.Module, ABC):
             masked_kspace = batched_mask_center(masked_kspace, pad, pad + num_low_freqs, mask_type=self.mask_type)
 
         # convert to image space
-        images, batches = self.chans_to_batch_dim(ifft2c(masked_kspace))
+        images, batches = self.chans_to_batch_dim(
+            ifft2(
+                masked_kspace,
+                centered=self.fft_centered,
+                normalization=self.fft_normalization,
+                spatial_dims=self.spatial_dims,
+            )
+        )
 
         # estimate sensitivities
         images = self.batch_chans_to_chan_dim(self.norm_unet(images), batches)
         if self.normalize:
-            images = self.divide_root_sum_of_squares(images)
+            images = self.divide_root_sum_of_squares(images, self.coil_dim)
         return images
