@@ -193,9 +193,9 @@ class Typing(ABC):
             if hasattr(value, "shape"):
                 value_shape = value.shape
                 type_shape = metadata.base_types[key].axes
-                name = key
-
                 if type_shape is not None and len(value_shape) != len(tuple(type_shape)):
+                    name = key
+
                     raise TypeError(
                         f"Input shape mismatch occurred for {name} in module {self.__class__.__name__} : \n"
                         f"Input shape expected = {metadata.base_types[name].axes} | \n"
@@ -251,18 +251,9 @@ class Typing(ABC):
         if metadata.is_singular_container_type:
             pass
 
-        # In all other cases, python will wrap multiple outputs into an outer tuple.
-        # Allow number of output arguments to be <= total output neural types and >= mandatory outputs.
-
         elif len(out_container) > len(out_types_list) or len(out_container) < len(mandatory_out_types_list):
             raise TypeError(
-                "Number of output arguments provided ({}) is not as expected. It should be larger than {} and "
-                "less than {}.\n"
-                "This can be either because insufficient/extra number of output NeuralTypes were provided,"
-                "or the provided NeuralTypes {} should enable container support "
-                "(add '[]' to the NeuralType definition)".format(
-                    len(out_container), len(out_types_list), len(mandatory_out_types_list), output_types
-                )
+                f"Number of output arguments provided ({len(out_container)}) is not as expected. It should be larger than {len(out_types_list)} and less than {len(mandatory_out_types_list)}.\nThis can be either because insufficient/extra number of output NeuralTypes were provided,or the provided NeuralTypes {output_types} should enable container support (add '[]' to the NeuralType definition)"
             )
 
             # Attach types recursively, if possible
@@ -302,8 +293,17 @@ class Typing(ABC):
             for ind, res in enumerate(out_objects):
                 self.__attach_neural_type(res, metadata, depth=0, name=out_types_list[ind][0])
 
-    def __check_neural_type(self, obj, metadata, depth, name=None):
-        """Checks if the object is of the correct type, and attaches the correct NeuralType."""
+    def __check_neural_type(self, obj, metadata, depth: int, name: str = None):
+        """
+        Checks if the object is of the correct type, and attaches the correct NeuralType.
+
+        Parameters
+        ----------
+        obj : Any python object that can be assigned to a value.
+        metadata : TypecheckMetadata object.
+        depth : Current depth of the recursion.
+        name : Optional name used of the source object, when an error is raised.
+        """
         if isinstance(obj, (tuple, list)):
             for elem in obj:
                 self.__check_neural_type(elem, metadata, depth + 1, name=name)
@@ -342,8 +342,17 @@ class Typing(ABC):
                     f"Input shape found : {value_shape}"
                 )
 
-    def __attach_neural_type(self, obj, metadata, depth, name=None):
-        """Attach NeuralType to the object."""
+    def __attach_neural_type(self, obj, metadata, depth: int, name: str = None):
+        """
+        Attach NeuralType to the object.
+
+        Parameters
+        ----------
+        obj : Any python object that can be assigned to a value.
+        metadata : TypecheckMetadata object.
+        depth : Current depth of the recursion.
+        name : Optional name used of the source object, when an error is raised.
+        """
         if isinstance(obj, (tuple, list)):
             for elem in obj:
                 self.__attach_neural_type(elem, metadata, depth=depth + 1, name=name)
@@ -411,30 +420,25 @@ class Serialization(ABC):
                 try:
                     # try to import the target class
                     imported_cls = mridc.utils.model_utils.import_class_by_path(target_cls)  # type: ignore
-
                     # use subclass instead
                     if issubclass(cls, imported_cls):
                         imported_cls = cls
-                        if accepts_trainer := Serialization._inspect_signature_for_trainer(imported_cls):
-                            if trainer is None:
-                                # Create a dummy PL trainer object
-                                cfg_trainer = TrainerConfig(
-                                    gpus=1, accelerator="ddp", num_nodes=1, logger=False, checkpoint_callback=False
-                                )
-                                trainer = Trainer(cfg_trainer)
-                            instance = imported_cls(cfg=config, trainer=trainer)  # type: ignore
-                        else:
-                            instance = imported_cls(cfg=config)  # type: ignore
+                    if accepts_trainer := Serialization._inspect_signature_for_trainer(imported_cls):
+                        # Create a dummy PL trainer object
+                        instance = imported_cls(cfg=config, trainer=trainer)  # type: ignore
+                    else:
+                        instance = imported_cls(cfg=config)  # type: ignore
 
                 except Exception as e:
                     tb = traceback.format_exc()
                     prev_error = f"Model instantiation failed.\nTarget class: {target_cls}\nError: {e}\n{tb}"
-
                     logging.debug(prev_error + "\n falling back to 'cls'.")
             # target class resolution was unsuccessful, fall back to current `cls`
             if instance is None:
                 try:
                     if accepts_trainer := Serialization._inspect_signature_for_trainer(cls):
+                        instance = cls(cfg=config, trainer=trainer)  # type: ignore
+                    else:
                         instance = cls(cfg=config)  # type: ignore
                 except Exception as e:
                     # report saved errors, if any, and raise the current error
@@ -469,7 +473,8 @@ class Serialization(ABC):
         """Inspects the signature of the class to see if it accepts a trainer argument."""
         if hasattr(check_cls, "__init__"):
             signature = inspect.signature(check_cls.__init__)
-            return Trainer in signature.parameters
+            if "trainer" in signature.parameters:
+                return True
         return False
 
 
@@ -477,7 +482,15 @@ class FileIO(ABC):
     """Base class for file IO."""
 
     def save_to(self, save_path: str):
-        """Saves module/model with weights"""
+        """
+        Standardized method to save a tarfile containing the checkpoint, config, and any additional artifacts.
+        Implemented via :meth:`mridc.core.connectors.save_restore_connector.SaveRestoreConnector.save_to`.
+
+        Parameters
+        ----------
+        save_path: Path to save the checkpoint to.
+            str
+        """
         raise NotImplementedError()
 
     @classmethod
@@ -491,7 +504,29 @@ class FileIO(ABC):
         trainer: Optional[Trainer] = None,
         save_restore_connector: SaveRestoreConnector = None,
     ):
-        """Restores module/model with weights"""
+        """
+        Restores model instance (weights and configuration) from a .mridc file.
+
+        Parameters
+        ----------
+        restore_path: Path to .mridc file from which model should be instantiated.
+            str
+        override_config_path: Path to .yaml file containing the configuration to override the one in the .mridc file.
+            str
+        map_location: Device to map the instantiated model to. By default (None), it will select a GPU if available, \
+        falling back to CPU otherwise.
+            torch.device
+        strict: Passed to load_state_dict. By default True.
+            bool
+        return_config: If True, returns the underlying config of the restored model as an OmegaConf DictConfig \
+        object without instantiating the model.
+            bool
+        trainer: If provided, will be used to instantiate the model.
+            Trainer
+        save_restore_connector: An optional SaveRestoreConnector object that defines the implementation of the \
+        restore_from() method.
+            SaveRestoreConnector
+        """
         raise NotImplementedError()
 
     @classmethod
@@ -653,7 +688,7 @@ class Model(Typing, Serialization, FileIO, ABC):  # type: ignore
         if location_in_the_cloud is None:
             raise FileNotFoundError(
                 f"Model {model_name} was not found. "
-                f"Check cls.list_available_models() for the list of all available models."
+                "Check cls.list_available_models() for the list of all available models."
             )
         filename = location_in_the_cloud.split("/")[-1]
         url = location_in_the_cloud.replace(filename, "")
@@ -681,7 +716,30 @@ class Model(Typing, Serialization, FileIO, ABC):  # type: ignore
 
 
 class typecheck:
-    """Decorator to check the type of the input arguments."""
+    """
+    A decorator which performs input-output neural type checks, and attaches neural types to the output of the
+    function that it wraps.
+    Requires that the class inherit from `mridc.core.Typing` in order to perform type checking, and will raise an
+    error if that is not the case.
+
+    # Usage (Class level type support)
+    .. code-block:: python
+
+        @typecheck()
+        def fn(self, arg1, arg2, ...):
+
+    # Usage (Function level type support)
+    .. code-block:: python
+
+        @typecheck(input_types=..., output_types=...)
+        def fn(self, arg1, arg2, ...):
+
+    Points to be noted:
+        1) The brackets () in `@typecheck()` are necessary. You will encounter a TypeError: __init__() takes 1 \
+        positional argument but X were given without those brackets.
+        2) The function can take any number of positional arguments during definition. When you call this function, \
+        all arguments must be passed using kwargs only.
+    """
 
     class TypeState(Enum):
         """
@@ -698,28 +756,6 @@ class typecheck:
         output_types: Union[TypeState, Optional[Dict[str, NeuralType]]] = TypeState.UNINITIALIZED,
         ignore_collections: bool = False,
     ):
-        """
-        A decorator which performs input-output neural type checks, and attaches neural types to the output of the
-        function that it wraps.
-        Requires that the class inherit from `mridc.core.Typing` in order to perform type checking, and will raise an
-        error if that is not the case.
-
-        # Usage (Class level type support)
-        @typecheck()
-        def fn(self, arg1, arg2, ...):
-            ...
-        # Usage (Function level type support)
-        @typecheck(input_types=..., output_types=...)
-        def fn(self, arg1, arg2, ...):
-            ...
-
-        Points to be noted:
-            1) The brackets () in `@typecheck()` are necessary.
-                You will encounter a TypeError: __init__() takes 1 positional argument but X
-                were given without those brackets.
-            2) The function can take any number of positional arguments during definition.
-                When you call this function, all arguments must be passed using kwargs only.
-        """
         self.input_types = input_types
         self.output_types = output_types
 
@@ -729,6 +765,11 @@ class typecheck:
 
     @wrapt.decorator(enabled=is_typecheck_enabled)
     def __call__(self, wrapped, instance: Typing, args, kwargs):
+        """
+        Wrapper method that can be used on any function of a class that implements :class:`~mridc.core.Typing`. By \
+        default, it will utilize the `input_types` and `output_types` properties of the class inheriting Typing. \
+        Local function level overrides can be provided by supplying dictionaries as arguments to the decorator.
+        """
         if instance is None:
             raise RuntimeError("Only classes which inherit mridc.core.Typing can use this decorator !")
 
