@@ -9,7 +9,6 @@ import sys
 import time
 from copy import deepcopy
 from dataclasses import dataclass
-
 from pathlib import Path
 from shutil import copy, move
 from typing import Any, Dict, List, Optional, Tuple, Union
@@ -306,15 +305,8 @@ def exp_manager(trainer: Trainer, cfg: Optional[Union[DictConfig, Dict]] = None)
     mridc_testing = get_envbool(MRIDC_ENV_VARNAME_TESTING, False)
 
     log_file = log_dir / f"mridc_log_globalrank-{global_rank}_localrank-{local_rank}.txt"
-    # Handle logging to file
-    # Logs local rank 0 only
-    if local_rank == 0 and cfg.log_local_rank_0_only and not mridc_testing:
-        logging.add_file_handler(log_file)
-    elif global_rank == 0 and cfg.log_global_rank_0_only and mridc_testing:
-        logging.add_file_handler(log_file)
-    else:
-        logging.add_file_handler(log_file)
 
+    logging.add_file_handler(log_file)
     # For some reason, LearningRateLogger requires trainer to have a logger. Safer to create logger on all ranks
     # not just global rank 0.
     if cfg.create_tensorboard_logger or cfg.create_wandb_logger:
@@ -600,7 +592,7 @@ def get_git_hash():
     try:
         return True, subprocess.check_output(["git", "rev-parse", "HEAD"], stderr=subprocess.STDOUT).decode()
     except subprocess.CalledProcessError as err:
-        return False, "{}\n".format(err.output.decode("utf-8"))
+        return False, f'{err.output.decode("utf-8")}\n'
 
 
 def get_git_diff():
@@ -615,7 +607,7 @@ def get_git_diff():
     try:
         return subprocess.check_output(["git", "diff"], stderr=subprocess.STDOUT).decode()
     except subprocess.CalledProcessError as err:
-        return "{}\n".format(err.output.decode("utf-8"))
+        return f'{err.output.decode("utf-8")}\n'
 
 
 class LoggerList(_LoggerCollection):
@@ -687,7 +679,10 @@ def configure_loggers(
             wandb_kwargs = {}
         if "name" not in wandb_kwargs and "project" not in wandb_kwargs:
             raise ValueError("name and project are required for wandb_logger")
-        wandb_logger = WandbLogger(save_dir=exp_dir[0], version=version, **wandb_kwargs)
+        # if wandb_kwargs.get("save_dir", None) is None:
+        #     wandb_kwargs["save_dir"] = str(exp_dir[0])
+        #     os.makedirs(wandb_kwargs["save_dir"], exist_ok=True)
+        wandb_logger = WandbLogger(save_dir=str(exp_dir[0]), version=version, **wandb_kwargs)
 
         logger_list.append(wandb_logger)
         logging.info("WandBLogger has been set up")
@@ -849,19 +844,31 @@ class MRIDCModelCheckpoint(ModelCheckpoint):
         if trainer.fast_dev_run:
             return None
 
+        # check if we need to save a last checkpoint manually as validation isn't always run based on the interval
+        if self.save_last and trainer.val_check_interval != 0:
+            should_save_last_checkpoint = False
+            if isinstance(trainer.val_check_interval, float) and trainer.val_check_interval % trainer.global_step != 0:
+                should_save_last_checkpoint = True
+            if isinstance(trainer.val_check_interval, int) and trainer.global_step % trainer.val_check_interval != 0:
+                should_save_last_checkpoint = True
+            if should_save_last_checkpoint:
+                monitor_candidates = self._monitor_candidates(trainer)
+                super()._save_last_checkpoint(trainer, monitor_candidates)
+
         # Call parent on_train_end() to save the -last checkpoint
         super().on_train_end(trainer, pl_module)
 
         # Load the best model and then re-save it
         if self.save_best_model:
             # wait for all processes to finish
-            trainer.training_type_plugin.barrier("SaveBestCheckpointConnector.resume_end")
+            trainer.strategy.barrier("SaveBestCheckpointConnector.resume_end")
             if self.best_model_path == "":
                 logging.warning(
                     f"{self} was told to save the best checkpoint at the end of training, but no saved checkpoints "
                     "were found. Saving latest model instead."
                 )
             else:
+                self.best_model_path = trainer.strategy.broadcast(self.best_model_path)
                 trainer._checkpoint_connector.restore(self.best_model_path)
 
         if self.save_mridc_on_train_end:
@@ -938,7 +945,7 @@ def configure_checkpointing(trainer: Trainer, log_dir: Path, name: str, resume:
                 f"{trainer.max_epochs}) was less than trainer.check_val_every_n_epoch("
                 f"{trainer.check_val_every_n_epoch}). It is very likely this run will fail with "
                 f"ModelCheckpoint(monitor='{params.monitor}') not found in the returned metrics. Please ensure that "
-                f"validation is run within trainer.max_epochs."
+                "validation is run within trainer.max_epochs."
             )
         elif trainer.max_steps is not None:
             logging.warning(
