@@ -162,16 +162,29 @@ class TestOptimizersSchedulers:
     INITIAL_LR = 0.1
     MIN_LR = 1e-3
     MAX_STEPS = 10
+    D_MODEL = 16
 
     # fused_adam is looking for CUDA and this test is being run on CPU only tests
     @pytest.mark.unit
     def test_get_optimizer(self):
         """Test that the optimizer is correctly created"""
         model = TempModel()
+        if torch.cuda.is_available():
+            model.cuda()
 
         for opt_name in AVAILABLE_OPTIMIZERS:
             if opt_name == "fused_adam" and not torch.cuda.is_available():
                 continue
+            if opt_name == "distributed_fused_adam":
+                if not torch.cuda.is_available() or not torch.distributed.is_nccl_available():
+                    continue
+                if not torch.distributed.is_initialized():
+                    torch.distributed.init_process_group(
+                        "nccl",
+                        world_size=1,
+                        rank=0,
+                        store=torch.distributed.HashStore(),
+                    )
             opt_cls = get_optimizer(opt_name)
             if opt_name == "adafactor":
                 # Adafactor's default mode uses relative_step without any lr.
@@ -713,6 +726,57 @@ class TestOptimizersSchedulers:
         if final_lr != self.MIN_LR:
             raise AssertionError
 
+    # Noam scheduler should decay past MAX_STEPS - run two schedulers in parallel to test it
+    @pytest.mark.unit
+    def test_NoamAnnealing(self):
+        model = TempModel()
+        opt_cls = get_optimizer("novograd")
+        opt1 = opt_cls(model.parameters(), lr=self.INITIAL_LR)
+        opt2 = opt_cls(model.parameters(), lr=self.INITIAL_LR)
+
+        # No warmup case
+        policy1 = optim.lr_scheduler.NoamAnnealing(
+            opt1, d_model=self.D_MODEL, max_steps=self.MAX_STEPS, min_lr=self.MIN_LR
+        )
+        policy2 = optim.lr_scheduler.NoamAnnealing(
+            opt2, d_model=self.D_MODEL, max_steps=self.MAX_STEPS * 2, min_lr=self.MIN_LR
+        )
+        initial_lr = policy1.get_last_lr()[0]
+
+        assert initial_lr == self.D_MODEL ** (-0.5) * self.INITIAL_LR
+
+        for _ in range(self.MAX_STEPS * 2):
+            if policy1.get_last_lr()[0] > self.INITIAL_LR:
+                raise AssertionError
+            assert policy1.get_last_lr()[0] <= policy2.get_last_lr()[0]
+            opt1.step()
+            opt2.step()
+            policy1.step()
+            policy2.step()
+
+        # Warmup steps available
+        policy1 = optim.lr_scheduler.NoamAnnealing(
+            opt1, d_model=self.D_MODEL, warmup_steps=5, max_steps=self.MAX_STEPS, min_lr=self.MIN_LR
+        )
+        policy2 = optim.lr_scheduler.NoamAnnealing(
+            opt2, d_model=self.D_MODEL, warmup_steps=5, max_steps=self.MAX_STEPS * 2, min_lr=self.MIN_LR
+        )
+        initial_lr = policy1.get_last_lr()[0]
+
+        assert initial_lr < self.INITIAL_LR
+
+        for i in range(self.MAX_STEPS * 2):
+            if i <= 5:
+                assert policy1.get_last_lr()[0] <= self.INITIAL_LR
+            else:
+                assert self.MIN_LR <= policy1.get_last_lr()[0] <= self.INITIAL_LR
+                assert policy1.get_last_lr()[0] <= policy2.get_last_lr()[0]
+
+            opt1.step()
+            opt2.step()
+            policy1.step()
+            policy2.step()
+
     @pytest.mark.unit
     def test_PolynomialDecayAnnealing(self):
         """Test PolynomialDecayAnnealing"""
@@ -939,28 +1003,25 @@ class TestOptimizersSchedulers:
     @pytest.mark.unit
     @pytest.mark.run_only_on("CPU")
     def test_max_step_computation(self):
-        """Test that the max_step computation is correct."""
+        """Test max step computation."""
 
         def train(
-            max_epochs, accumulate_grad_batches, limit_train_batches, num_processes, batch_size, dataset_len, drop_last
+            max_epochs, accumulate_grad_batches, limit_train_batches, devices, batch_size, dataset_len, drop_last
         ):
-            """Set up the training environment"""
             trainer = pl.Trainer(
                 max_epochs=max_epochs,
                 strategy="ddp_spawn",
                 accelerator="cpu",
-                num_processes=num_processes,
+                devices=devices,
                 accumulate_grad_batches=accumulate_grad_batches,
                 limit_train_batches=limit_train_batches,
                 enable_checkpointing=False,
-                progress_bar_refresh_rate=0,
-                weights_summary=None,
             )
             max_steps = optim.lr_scheduler.compute_max_steps(
                 max_epochs,
                 accumulate_grad_batches,
                 limit_train_batches,
-                num_processes,
+                devices,
                 dataset_len,
                 batch_size,
                 drop_last,
@@ -974,7 +1035,7 @@ class TestOptimizersSchedulers:
             31,
             accumulate_grad_batches=1,
             limit_train_batches=1.0,
-            num_processes=9,
+            devices=9,
             batch_size=60,
             dataset_len=1613,
             drop_last=True,
@@ -982,8 +1043,8 @@ class TestOptimizersSchedulers:
         train(
             5,
             accumulate_grad_batches=1,
-            limit_train_batches=0.17382691901706027,
-            num_processes=4,
+            limit_train_batches=1.0,
+            devices=4,
             batch_size=97,
             dataset_len=498,
             drop_last=False,
@@ -991,8 +1052,8 @@ class TestOptimizersSchedulers:
         train(
             5,
             accumulate_grad_batches=8,
-            limit_train_batches=0.1663306588594945,
-            num_processes=4,
+            limit_train_batches=1.0,
+            devices=4,
             batch_size=54,
             dataset_len=629,
             drop_last=True,
@@ -1000,8 +1061,8 @@ class TestOptimizersSchedulers:
         train(
             5,
             accumulate_grad_batches=1,
-            limit_train_batches=0.2121376533631948,
-            num_processes=1,
+            limit_train_batches=1.0,
+            devices=1,
             batch_size=68,
             dataset_len=488,
             drop_last=False,
@@ -1011,19 +1072,17 @@ class TestOptimizersSchedulers:
             accumulate_grad_batches = random.randint(1, 10)
 
             limit_train_batches_int = random.randint(1, 10)
-            limit_train_batches_float = random.uniform(0, 1)
+            limit_train_batches_float = 1.0
             limit_train_batches = random.choice([limit_train_batches_int, limit_train_batches_float])
             max_epochs = random.randint(4, 20)
-            num_processes = random.randint(1, 5)
-            dataset_len = random.randint(20, num_processes * 500)
-            batch_size = random.randint(
-                math.ceil(5.0 / num_processes), min(np.floor_divide(dataset_len, num_processes), 128)
-            )
+            devices = random.randint(1, 5)
+            dataset_len = random.randint(20, devices * 500)
+            batch_size = random.randint(math.ceil(5.0 / devices), min(dataset_len // devices, 128))
             train(
                 max_epochs,
                 accumulate_grad_batches,
                 limit_train_batches,
-                num_processes,
+                devices,
                 batch_size,
                 dataset_len,
                 drop_last,
@@ -1046,8 +1105,6 @@ class TestOptimizersSchedulers:
                 accumulate_grad_batches=accumulate_grad_batches,
                 limit_train_batches=limit_train_batches,
                 enable_checkpointing=False,
-                progress_bar_refresh_rate=0,
-                weights_summary=None,
             )
             model = ExampleModel(batch_size, dataset_len, drop_last, max_steps)
             trainer.callbacks.append(SchedulerNoOpCallback())
